structure PosixError: POSIX_ERROR_EXTRA =
   struct
      open PosixPrimitive.Error

      exception SysErr of string * syserror option

      fun toWord a = Word.fromInt a
      fun fromWord a = Word.toInt a

      fun errorMsg (n: int) : string =
	 let val s = strerror n
	 in if PosixPrimitive.CString.isNull s
	       then "Unknown error"
	    else PosixPrimitive.CString.toString s
	 end

      fun raiseSys n = raise SysErr (errorMsg n, SOME n)

      fun restart (f: 'a -> int) (a: 'a): int =
	 let
	    fun loop () =
	       case f a of
		  ~1 => let val errno = getErrno ()
			in if errno = intr
			      then loop ()
			   else raiseSys errno
			end
		| n => n
	 in loop ()
	 end
      
      fun error () = raiseSys (getErrno ())

      fun checkReturnResult (n: int) = if n = ~1 then error () else n
      fun checkResult n = (checkReturnResult n; ())
   end
