// Template for PosixPrimitive.sml

#include "posix-constants.h"
#include "posix-enum.h"

#define WORD_XTR(X)   0w ## X
#define WORD(X)       WORD_XTR(X)

structure PosixPrimitive =
   struct
      type fd = int
      type pid = int
      type uid = word
      type gid = word
      type signal = int
      type size = int
      type ssize = int
      type mode = word
      type time = int
      type file_desc = int

      structure CString :> sig
			     type cstring
			     val isNull : cstring -> bool
			     val toString : cstring -> string
			   end =
         struct
	   type cstring = string
	   fun isNull(s : string) : bool = prim("__is_null", s)
	   fun toString x = x
	 end       
      type cstring = CString.cstring

      structure ForeignPtr : sig
			       type foreignptr
			       val isNull : foreignptr -> bool
			     end
			  =
	 struct
	   type foreignptr = foreignptr
	   fun toInt (fp:foreignptr):int = prim("id",fp)
	   fun isNull (fp:foreignptr) : bool = (toInt fp = 0)
	 end

      structure Error =
	 struct
	    type syserror = int

	    fun getErrno () : syserror = 
	      prim("Posix_Error_getErrno", ())

	    fun clearErrno () : unit = 
	      prim("Posix_Error_clearErrno", ())

	    fun strerror (e:syserror) : cstring = 
	      prim("Posix_Error_strerror", e)

	    val acces = Posix_Error_acces: syserror
	    val again = Posix_Error_again: syserror
	    val badf = Posix_Error_badf: syserror
	    val badmsg = Posix_Error_badmsg: syserror
	    val busy = Posix_Error_busy: syserror
	    val canceled = Posix_Error_canceled: syserror
	    val child = Posix_Error_child: syserror
	    val deadlk = Posix_Error_deadlk: syserror
	    val dom = Posix_Error_dom: syserror
	    val exist = Posix_Error_exist: syserror
	    val fault = Posix_Error_fault: syserror
	    val fbig = Posix_Error_fbig: syserror
	    val inprogress = Posix_Error_inprogress: syserror
	    val intr = Posix_Error_intr: syserror
	    val inval = Posix_Error_inval: syserror
	    val io = Posix_Error_io: syserror
	    val isdir = Posix_Error_isdir: syserror
	    val loop = Posix_Error_loop: syserror
	    val mfile = Posix_Error_mfile: syserror
	    val mlink = Posix_Error_mlink: syserror
	    val msgsize = Posix_Error_msgsize: syserror
	    val nametoolong = Posix_Error_nametoolong: syserror
	    val nfile = Posix_Error_nfile: syserror
	    val nodev = Posix_Error_nodev: syserror
	    val noent = Posix_Error_noent: syserror
	    val noexec = Posix_Error_noexec: syserror
	    val nolck = Posix_Error_nolck: syserror
	    val nomem = Posix_Error_nomem: syserror
	    val nospc = Posix_Error_nospc: syserror
	    val nosys = Posix_Error_nosys: syserror
	    val notdir = Posix_Error_notdir: syserror
	    val notempty = Posix_Error_notempty: syserror
	    val notsup = Posix_Error_notsup: syserror
	    val notty = Posix_Error_notty: syserror
	    val nxio = Posix_Error_nxio: syserror
	    val perm = Posix_Error_perm: syserror
	    val pipe = Posix_Error_pipe: syserror
	    val range = Posix_Error_range: syserror
	    val rofs = Posix_Error_rofs: syserror
	    val spipe = Posix_Error_spipe: syserror
	    val srch = Posix_Error_srch: syserror
	    val toobig = Posix_Error_toobig: syserror
	    val xdev = Posix_Error_xdev: syserror

	    fun errorName (n:syserror) : string =
	      case n
		of EACCES => "acces"
		 | EAGAIN => "again"
		 | EBADF => "badf"
		 | EBADMSG => "badmsg"
		 | EBUSY => "busy"
		 | ECANCELED => "canceled"
		 | ECHILD => "child"
		 | EDEADLK => "deadlk"
		 | EDOM => "dom"
		 | EEXIST => "exist"
		 | EFAULT => "fault"
		 | EFBIG => "fbig"
		 | EINPROGRESS => "inprogress"
		 | EINTR => "intr"
		 | EINVAL => "inval"
		 | EIO => "io"
		 | EISDIR => "isdir"
		 | ELOOP => "loop"
		 | EMFILE => "mfile"
		 | EMLINK => "mlink"
		 | EMSGSIZE => "msgsize"
		 | ENAMETOOLONG => "nametoolong"
		 | ENFILE => "nfile"
		 | ENODEV => "nodev"
		 | ENOENT => "noent"
		 | ENOEXEC => "noexec"
		 | ENOLCK => "nolck"
		 | ENOMEM => "nomem"
		 | ENOSPC => "nospc"
		 | ENOSYS => "nosys"
		 | ENOTDIR => "notdir"
		 | ENOTEMPTY => "notempty"
		 | ENOTSUP => "notsup"
		 | ENOTTY => "notty"
		 | ENXIO => "nxio"
		 | EPERM => "perm"
		 | EPIPE => "pipe"
		 | ERANGE => "range"
		 | EROFS => "rofs"
		 | ESPIPE => "spipe"
		 | ESRCH => "srch"
		 | E2BIG => "toobig"
		 | EXDEV => "xdev"
		 | _ => "<UNKNOWN>"
		  
	    fun syserror (s:string) : syserror option =
	      case s
		of "acces" => SOME EACCES
		 | "again" => SOME EAGAIN
		 | "badf" => SOME EBADF
		 | "badmsg" => SOME EBADMSG
		 | "busy" => SOME EBUSY
		 | "canceled" => SOME ECANCELED
		 | "child" => SOME ECHILD
		 | "deadlk" => SOME EDEADLK
		 | "dom" => SOME EDOM
		 | "exist" => SOME EEXIST
		 | "fault" => SOME EFAULT
		 | "fbig" => SOME EFBIG
		 | "inprogress" => SOME EINPROGRESS
		 | "intr" => SOME EINTR
		 | "inval" => SOME EINVAL
		 | "io" => SOME EIO
		 | "isdir" => SOME EISDIR
		 | "loop" => SOME ELOOP
		 | "mfile" => SOME EMFILE
		 | "mlink" => SOME EMLINK
		 | "msgsize" => SOME EMSGSIZE
		 | "nametoolong" => SOME ENAMETOOLONG
		 | "nfile" => SOME ENFILE
		 | "nodev" => SOME ENODEV
		 | "noent" => SOME ENOENT
		 | "noexec" => SOME ENOEXEC
		 | "nolck" => SOME ENOLCK
		 | "nomem" => SOME ENOMEM
		 | "nospc" => SOME ENOSPC
		 | "nosys" => SOME ENOSYS
		 | "notdir" => SOME ENOTDIR
		 | "notempty" => SOME ENOTEMPTY
		 | "notsup" => SOME ENOTSUP
		 | "notty" => SOME ENOTTY
		 | "nxio" => SOME ENXIO
		 | "perm" => SOME EPERM
		 | "pipe" => SOME EPIPE
		 | "range" => SOME ERANGE
		 | "rofs" => SOME EROFS
		 | "spipe" => SOME ESPIPE
		 | "srch" => SOME ESRCH
		 | "toobig" => SOME E2BIG
		 | "xdev" => SOME EXDEV
		 | _ => NONE
	 end
      
      structure FileSys =
	 struct
	    type file_desc = file_desc

	    type ino = int
	    type dev = word
	    type uid = uid
	    type gid = gid

	    structure S =
	       struct
		  type mode = word
		  val ifsock = Posix_FileSys_S_ifsock_VAL: mode (* Posix_FileSys_S_ifsock *)
		  val iflnk = Posix_FileSys_S_iflnk_VAL: mode (* Posix_FileSys_S_iflnk *)
		  val ifreg = Posix_FileSys_S_ifreg_VAL: mode (* Posix_FileSys_S_ifreg *)
		  val ifblk = Posix_FileSys_S_ifblk_VAL: mode (* Posix_FileSys_S_ifblk *)
		  val ifdir = Posix_FileSys_S_ifdir_VAL: mode (* Posix_FileSys_S_ifdir *)
		  val ifchr = Posix_FileSys_S_ifchr_VAL: mode (* Posix_FileSys_S_ifchr *)
		  val ififo = Posix_FileSys_S_ififo_VAL: mode (* Posix_FileSys_S_ififo *)
		  val irwxu = Posix_FileSys_S_irwxu_VAL: mode (* Posix_FileSys_S_irwxu *)
		  val irusr = Posix_FileSys_S_irusr_VAL: mode (* Posix_FileSys_S_irusr *)
		  val iwusr = Posix_FileSys_S_iwusr_VAL: mode (* Posix_FileSys_S_iwusr *)
		  val ixusr = Posix_FileSys_S_ixusr_VAL: mode (* Posix_FileSys_S_ixusr *)
		  val irwxg = Posix_FileSys_S_irwxg_VAL: mode (* Posix_FileSys_S_irwxg *)
		  val irgrp = Posix_FileSys_S_irgrp_VAL: mode (* Posix_FileSys_S_irgrp *)
		  val iwgrp = Posix_FileSys_S_iwgrp_VAL: mode (* Posix_FileSys_S_iwgrp *)
		  val ixgrp = Posix_FileSys_S_ixgrp_VAL: mode (* Posix_FileSys_S_ixgrp *)
		  val irwxo = Posix_FileSys_S_irwxo_VAL: mode (* Posix_FileSys_S_irwxo *)
		  val iroth = Posix_FileSys_S_iroth_VAL: mode (* Posix_FileSys_S_iroth *)
		  val iwoth = Posix_FileSys_S_iwoth_VAL: mode (* Posix_FileSys_S_iwoth *)
		  val ixoth = Posix_FileSys_S_ixoth_VAL: mode (* Posix_FileSys_S_ixoth *)
		  val isuid = Posix_FileSys_S_isuid_VAL: mode (* Posix_FileSys_S_isuid *)
		  val isgid = Posix_FileSys_S_isgid_VAL: mode (* Posix_FileSys_S_isgid *)
	       end

	    structure O =
	       struct
		  type flags = word
		  val append = Posix_FileSys_O_append_VAL: flags (* Posix_FileSys_O_append *)
		  val creat = Posix_FileSys_O_creat_VAL: flags (* Posix_FileSys_O_creat *)
		  val excl = Posix_FileSys_O_excl_VAL: flags (* Posix_FileSys_O_excl *)
		  val noctty = Posix_FileSys_O_noctty_VAL: flags (* Posix_FileSys_O_noctty *)
		  val nonblock = Posix_FileSys_O_nonblock_VAL: flags (* Posix_FileSys_O_nonblock *)
		  val sync = Posix_FileSys_O_sync_VAL: flags (* Posix_FileSys_O_sync *)
		  val trunc = Posix_FileSys_O_trunc_VAL: flags (* Posix_FileSys_O_trunc *)
	       end

	    val o_rdonly = Posix_FileSys_o_rdonly_VAL: word (* Posix_FileSys_o_rdonly *)
	    val o_wronly = Posix_FileSys_o_wronly_VAL: word (* Posix_FileSys_o_wronly *)
	    val o_rdwr = Posix_FileSys_o_rdwr_VAL: word (* Posix_FileSys_o_rdwr *)
	    val R_OK_ = Posix_FileSys_R_OK_VAL: word (* Posix_FileSys_R_OK *)
	    val W_OK_ = Posix_FileSys_W_OK_VAL: word (* Posix_FileSys_W_OK *)
	    val X_OK_ = Posix_FileSys_X_OK_VAL: word (* Posix_FileSys_X_OK *)
	    val F_OK_ = Posix_FileSys_F_OK_VAL: word (* Posix_FileSys_F_OK *)

	    fun propertyLookup (s:string) : int option =
	      case s
		of "LINK_MAX" => SOME Posix_FileSys_LINK_MAX
		 | "MAX_CANON" => SOME Posix_FileSys_MAX_CANON
		 | "MAX_INPUT" => SOME Posix_FileSys_MAX_INPUT
		 | "NAME_MAX" => SOME Posix_FileSys_NAME_MAX
		 | "PATH_MAX" => SOME Posix_FileSys_PATH_MAX
		 | "PIPE_BUF" => SOME Posix_FileSys_PIPE_BUF
		 | "CHOWN_RESTRICTED" => SOME Posix_FileSys_CHOWN_RESTRICTED
		 | "NO_TRUNC" => SOME Posix_FileSys_NO_TRUNC
		 | "VDISABLE" => SOME Posix_FileSys_VDISABLE
		 | _ => NONE

	    structure Dirstream =
	       struct
		  type dirstream = foreignptr (*built-in type*)

		  fun closedir (ds:dirstream) : int = prim("@closedir", ds)
		  fun opendir (s:string) : dirstream = prim("@opendir", s)
		  fun readdir (ds:dirstream) : cstring =
		    prim("Posix_FileSys_Dirstream_readdir", ds)
		  fun rewinddir (ds:dirstream) : unit = prim("@rewinddir", ds)
	       end

	    structure Stat =
	       struct
		  fun dev():dev = prim("Posix_FileSys_Stat_dev", ())
		  fun ino():ino = prim("Posix_FileSys_Stat_ino", ())
		  fun mode():word = prim("Posix_FileSys_Stat_mode", ())
		  fun nlink():int = prim("Posix_FileSys_Stat_nlink", ())
		  fun uid():uid = prim("Posix_FileSys_Stat_uid", ())
		  fun gid():gid = prim("Posix_FileSys_Stat_gid", ())
		  fun size():int = prim("Posix_FileSys_Stat_size", ())
		  fun atime():time = prim("Posix_FileSys_Stat_atime", ())
		  fun mtime():time = prim("Posix_FileSys_Stat_mtime", ())
		  fun ctime():time = prim("Posix_FileSys_Stat_ctime", ())
		  fun fstat(fd:fd):int = prim("Posix_FileSys_Stat_fstat", fd)
		  fun lstat(s:string):int = prim("Posix_FileSys_Stat_lstat", s)
		  fun stat(s:string):int = prim("Posix_FileSys_Stat_stat", s)
	       end

	    structure Utimbuf =
	       struct
		 fun setActime(t:time) : unit =
		   prim("Posix_FileSys_Utimbuf_setActime", t)
		 fun setModtime(t:time) : unit =
		   prim("Posix_FileSys_Utimbuf_setModTime", t)
		 fun utime(s:string):int =
		   prim("Posix_FileSys_Utimbuf_utime", s)
	       end

	    fun access (s:string, w:word) : int = prim("@access", (s,w))
	    fun chdir (s:string): int = prim("@chdir", s)
	    fun chmod (s:string,m:mode) : int = prim("@chmod", (s,m))
	    fun chown (s:string,u:uid,g:gid): int = prim("@chown", (s,u,g))
	    fun fchmod (fd:fd,m:mode):int = prim("@fchmod", (fd,m))
	    fun fchown (fd:fd, u:uid, g:gid): int = prim("@fchown", (fd,u,g))
	    fun fpathconf (fd:fd,i:int):int = prim("@fpathconf", (fd,i))
	    fun ftruncate (fd:fd,i:int):int = prim("@ftruncate", (fd,i))
	    fun getcwd ():string = prim("Posix_FileSys_getcwd", ())
	    fun link {old:string,new:string} :int = prim("@link", (old,new))
	    fun mkdir (s:string, w:word) : int = prim("@mkdir", (s,w))
	    fun mkfifo (s:string, w:word): int = prim("@mkfifo", (s,w))
	    fun openn (s:string,w:word,m:mode): fd = prim("@open", (s,w,m))
	    fun pathconf (s:string,i:int):int = prim("@pathconf", (s,i))
	    fun readlink (s:string):cstring =
	      prim("Posix_FileSys_readlink", s)
	    fun rename {old:string,new:string} :int = prim("@rename", (old,new))
	    fun rmdir (s:string):int = prim("@rmdir", s)
	    fun symlink {old:string,new:string}: int = prim("@symlink", (old,new))
	    fun umask (w:word) : word = prim("@umask", w)
	    fun unlink (s:string):int = prim("@unlink", s)

	    structure ST =
	       struct
		 fun isDir(w:word):bool = prim("Posix_FileSys_ST_isDir", w)
		 fun isChr(w:word):bool = prim("Posix_FileSys_ST_isChr", w)
		 fun isBlk(w:word):bool = prim("Posix_FileSys_ST_isBlk", w)
		 fun isReg(w:word):bool = prim("Posix_FileSys_ST_isReg", w)
		 fun isFIFO(w:word):bool = prim("Posix_FileSys_ST_isFIFO", w)
		 fun isLink(w:word):bool = prim("Posix_FileSys_ST_isLink", w)
		 fun isSock(w:word):bool = prim("Posix_FileSys_ST_isSock", w)
	       end
	 end

      structure IO =
	 struct
	    val F_DUPFD_ = Posix_IO_F_DUPFD: int
	    val F_GETFD_ = Posix_IO_F_GETFD: int
	    val F_SETFD_ = Posix_IO_F_SETFD: int
	    val F_GETFL_ = Posix_IO_F_GETFL: int
	    val F_SETFL_ = Posix_IO_F_SETFL: int
	    val F_GETLK_ = Posix_IO_F_GETLK: int
	    val F_SETLK_ = Posix_IO_F_SETLK: int
	    val F_RDLCK_ = Posix_IO_F_RDLCK: int
	    val F_WRLCK_ = Posix_IO_F_WRLCK: int
	    val F_UNLCK_ = Posix_IO_F_UNLCK: int
	    val F_SETLKW_ = Posix_IO_F_SETLKW: int
	    val F_GETOWN_ = Posix_IO_F_GETOWN: int
	    val F_SETOWN_ = Posix_IO_F_SETOWN: int
	    val O_ACCMODE_ = Posix_IO_O_ACCMODE_VAL: word (* Posix_IO_O_ACCMODE *)
	    val SEEK_SET_ = Posix_IO_SEEK_SET: int
	    val SEEK_CUR_ = Posix_IO_SEEK_CUR: int
	    val SEEK_END_ = Posix_IO_SEEK_END: int

	    structure FD =
	       struct
		  type flags = word
		  val cloexec = WORD(Posix_IO_FD_cloexec): flags
	       end
	    
	    type file_desc = file_desc
	    type pid = pid

	    structure FLock =
	       struct
		 fun fcntl (fd:fd,i:int) : int = prim("Posix_IO_FLock_fcntl", (fd,i))
		 fun typ () : int = prim("Posix_IO_FLock_type", ())
		 fun whence () : int = prim("Posix_IO_FLock_whence", ())
		 fun start () : int = prim("Posix_IO_FLock_start", ())
		 fun len () : int = prim("Posix_IO_FLock_len", ())
		 fun pid () : int = prim("Posix_IO_FLock_pid", ())
		 fun setType (i:int) : unit = prim("Posix_IO_FLock_setType", i)
		 fun setWhence (i:int) : unit = prim("Posix_IO_FLock_setWhence", i)
		 fun setStart (i:int) : unit = prim("Posix_IO_FLock_setStart", i)
		 fun setLen (i:int) : unit = prim("Posix_IO_FLock_setLen", i)
		 fun setPid (i:int) : unit = prim("Posix_IO_FLock_setPid", i)
	       end
	    
	    fun close (fd:fd) : int = prim("@close", fd)
	    fun dup (fd:fd) : fd = prim("@dup", fd)
	    fun dup2 (fd1:fd, fd2:fd) : fd = prim("@dup2", (fd1,fd2))
	    fun fcntl2 (fd:fd,i:int) : int = prim("@fcntl", (fd,i))
	    fun fcntl3 (fd:fd,i1:int,i2:int) : int = prim("@fcntl", (fd,i1,i2))
	    fun fsync (fd:fd) : int = prim("@fsync", fd)
	    fun lseek (fd:fd,i1:int,i2:int) : int= prim("@lseek", (fd,i1,i2))
	      
	    local
	      type pipe = {infd:fd,outfd:fd}
	      fun isNull (p: pipe) : bool = prim("__is_null", p)
	      fun pipe0 () : pipe = prim("Posix_IO_pipe", ())
	    in
	      fun pipe () : pipe option = 
		let val p = pipe0()
		in if isNull p then NONE
		   else SOME p
		end
	    end

	    fun readArr (fd:fd,buf:chararray,i:int,sz:size) : ssize = 
	      prim("Posix_IO_read", (fd,buf,i,sz))

	    local
	      fun alloc (n:int) : Word8Vector.vector = 
		prim("allocStringML", n)
	      fun read (fd:fd,buf:Word8Vector.vector,i:int,sz:size) : ssize = 
		prim("Posix_IO_read", (fd,buf,i,sz))
	      fun update_unsafe(s:Word8Vector.vector,i:int,c:char) : unit = 
		prim("__bytetable_update", (s, i, c))
	    in
	      fun readVec(fd:fd, n:int) : Word8Vector.vector option =
		let val buf = alloc n
		  val i = read(fd, buf, 0, n)
		in if i = ~1 then NONE
		   else if i = n then (update_unsafe(buf,n,#"\000"); SOME buf)
			else SOME (Word8Vector.extract(buf,0,SOME i))
		end 
	    end

	    fun writeArr (fd:fd,buf:chararray,i:int,sz:size) : ssize = 
	      prim("Posix_IO_write", (fd,buf,i,sz))

	    fun writeVec (fd:fd,buf:Word8Vector.vector,i:int,sz:size) : ssize = 
	      prim("Posix_IO_write", (fd,buf,i,sz))
	 end	       

      structure ProcEnv =
	 struct
	    val numgroups = Posix_ProcEnv_numgroups: int
	    fun sysconfNameLookup s : int option =
	      case s
		of "ARG_MAX" => SOME Posix_ProcEnv_ARG_MAX
		 | "CHILD_MAX" => SOME Posix_ProcEnv_CHILD_MAX
		 | "CLK_TCK" => SOME Posix_ProcEnv_CLK_TCK
		 | "STREAM_MAX" => SOME Posix_ProcEnv_STREAM_MAX
		 | "TZNAME_MAX" => SOME Posix_ProcEnv_TZNAME_MAX
		 | "OPEN_MAX" => SOME Posix_ProcEnv_OPEN_MAX
		 | "JOB_CONTROL" => SOME Posix_ProcEnv_JOB_CONTROL
		 | "SAVED_IDS" => SOME Posix_ProcEnv_SAVED_IDS
		 | "VERSION" => SOME Posix_ProcEnv_VERSION
		 | "BC_BASE_MAX" => SOME Posix_ProcEnv_BC_BASE_MAX
		 | "BC_DIM_MAX" => SOME Posix_ProcEnv_BC_DIM_MAX
		 | "BC_SCALE_MAX" => SOME Posix_ProcEnv_BC_SCALE_MAX
		 | "BC_STRING_MAX" => SOME Posix_ProcEnv_BC_STRING_MAX
		 | "COLL_WEIGHTS_MAX" => SOME Posix_ProcEnv_COLL_WEIGHTS_MAX
		 | "EXPR_NEST_MAX" => SOME Posix_ProcEnv_EXPR_NEST_MAX
		 | "LINE_MAX" => SOME Posix_ProcEnv_LINE_MAX
		 | "RE_DUP_MAX" => SOME Posix_ProcEnv_RE_DUP_MAX
		 | "2_VERSION" => SOME Posix_ProcEnv_2_VERSION
		 | "2_FORT_DEV" => SOME Posix_ProcEnv_2_FORT_DEV
		 | "2_FORT_RUN" => SOME Posix_ProcEnv_2_FORT_RUN
		 | "2_SW_DEV" => SOME Posix_ProcEnv_2_SW_DEV
		 | _ => NONE

	    type pid = pid
	    type gid = gid
	    type uid = uid
	    type file_desc = file_desc

	    fun getegid () : gid = prim("@getegid", ())
	    fun geteuid () : uid = prim("@geteuid", ())
	    fun getgid () : gid = prim("@getgid", ())

	    fun getgroups () : gid list = prim("Posix_ProcEnv_getgroups", ())

	    fun getlogin () : cstring = prim("Posix_ProcEnv_getlogin", ())

	    fun getpgrp () : pid = prim("@getpgrp", ())
	    fun getpid () : pid = prim("@getpid", ())
	    fun getppid () : pid = prim("@getppid", ())
	    fun getuid () : uid = prim("@getuid", ())
	    fun setenv (s1:string,s2:string) : int = prim("@setenv", (s1,s2,1))
	    fun setgid (gid:gid) : int = prim("@setgid", gid)
	    fun setpgid (p1:pid,p2:pid) : int = prim("@setpgid", (p1,p2))
	    fun setsid () : pid = prim("@setsid", ())
	    fun setuid (uid:uid) : int = prim("@setuid", uid)
	      
	    structure Uname =
	      struct
		 (* type uname = pointer *)
		
		fun uname () : int = prim("Posix_ProcEnv_Uname_uname", ())
		fun sysname () : string = prim("Posix_ProcEnv_Uname_sysname", ())
		fun nodename () : string = prim("Posix_ProcEnv_Uname_nodename", ())
		fun release () : string = prim("Posix_ProcEnv_Uname_release", ())
		fun version () : string = prim("Posix_ProcEnv_Uname_version", ())
		fun machine () : string = prim("Posix_ProcEnv_Uname_machine", ())
	      end

	    type clock_t = word
	       
	    structure Tms =
	       struct
		  fun utime () : clock_t = prim("Posix_ProcEnv_Tms_utime", ())
		  fun stime () : clock_t = prim("Posix_ProcEnv_Tms_stime", ())
		  fun cutime () : clock_t = prim("Posix_ProcEnv_Tms_cutime", ())
		  fun cstime () : clock_t = prim("Posix_ProcEnv_Tms_cstime", ())
	       end

	    fun ctermid () : string = prim("Posix_ProcEnv_ctermid", ())
	    fun environ () : string list = rev(prim("Posix_ProcEnv_environ", ()))
	    fun getenv (s:string) : cstring = prim("Posix_ProcEnv_getenv", s)
	    fun isatty (fd:fd) : bool = prim("@isatty", fd)
	    fun sysconf (i:int) : int = prim("@sysconf", i)
	    fun times () : clock_t = prim("Posix_ProcEnv_times", ())
	    fun ttyname (fd:fd) : cstring = prim("Posix_ProcEnv_ttyname", fd)
	 end 

      structure Process =
	 struct
	    val wnohang = Posix_Process_wnohang_VAL: word (* Posix_Process_wnohang *)
	    structure W =
	       struct
		  type flags = word
		  val untraced = Posix_Process_W_untraced_VAL: flags (* Posix_Process_W_untraced *)
	       end
	    
	    type pid = pid
	    type status = int

	    fun alarm (i:int) : int = prim("@alarm", i)
	    fun exec (s:string,e:string list) : int =
	      prim("Posix_Process_exec", (s,e))
	    fun exece (s:string,l1:string list,l2:string list) : int =
	      prim("Posix_Process_exece", (s, l1, l2))
	    fun execp (s:string,e:string list) : int =
	       prim("Posix_Process_execp", (s,e))
	    fun exit (i:int) : unit = prim("@exit", i)
	    fun exitStatus (s:status) : int = 
	      prim("Posix_Process_exitStatus", s)
	    fun fork () : pid = prim("@fork", ())
	    fun ifExited (s:status) : bool = prim("Posix_Process_ifExited", s)
	    fun ifSignaled (s:status) : bool = prim("Posix_Process_ifSignaled", s)
	    fun ifStopped (s:status) : bool = prim("Posix_Process_ifStopped", s)
	    fun kill (pid:pid,s:signal) : int = prim("@kill", (pid,s))
	    fun pause () : int = prim("@pause", ())
	    fun sleep (i:int) : int = prim("@sleep", i)
	    fun stopSig (s:status) : signal = prim("Posix_Process_stopSig", s)
	    fun termSig (s:status) : signal = prim("Posix_Process_termSig", s)
            fun waitpid (p:pid,i:int) : pid * status = prim("Posix_Process_waitpid", (p,i))
	 end
      
      structure Signal =
	 struct
	    type signal = signal
      	    type how = int

	    val abrt = Posix_Signal_abrt: signal
	    val alrm = Posix_Signal_alrm: signal
	    val bus = Posix_Signal_bus: signal
	    val chld = Posix_Signal_chld: signal
	    val cont = Posix_Signal_cont: signal
	    val fpe = Posix_Signal_fpe: signal
	    val hup = Posix_Signal_hup: signal
	    val ill = Posix_Signal_ill: signal
	    val int = Posix_Signal_int: signal
	    val kill = Posix_Signal_kill: signal
	    val pipe = Posix_Signal_pipe: signal
	    val prof = Posix_Signal_prof: signal
	    val quit = Posix_Signal_quit: signal
	    val segv = Posix_Signal_segv: signal
	    val stop = Posix_Signal_stop: signal
	    val term = Posix_Signal_term: signal
	    val tstp = Posix_Signal_tstp: signal
	    val ttin = Posix_Signal_ttin: signal
	    val ttou = Posix_Signal_ttou: signal
	    val usr1 = Posix_Signal_usr1: signal
	    val usr2 = Posix_Signal_usr2: signal
	    val vtalrm = Posix_Signal_vtalrm: signal
	       
	    val block = Posix_Signal_block: how
(*
	    fun default (s:signal) : int = 
	      prim("Posix_Signal_default", s)
	    fun handlee (s:signal) : int = 
	      prim("Posix_Signal_handle", s)
	    fun ignore (s:signal) : int = 
	      prim("Posix_Signal_ignore", s)
	    fun isDefault (s:signal) : bool = 
	      prim("Posix_Signal_isDefault", s)
	    fun isPending (s:signal) : bool = 
	      prim("Posix_Signal_isPending", s)
*)
	    val numSignals = Posix_Signal_numSignals: int
	    val setmask = Posix_Signal_setmask: how
(*
	    fun sigaddset (s:signal) : int = 
	      prim("Posix_Signal_sigaddset", s)
	    fun sigemptyset () : int = 
	      prim("Posix_Signal_sigemptyset", ())
	    fun sigfillset () : int = 
	      prim("Posix_Signal_sigfillset", ())
	    fun sigprocmask (h:how) : int = 
	      prim("Posix_Signal_sigprocmask", h)
*)
	    val unblock = Posix_Signal_unblock: how
	 end
      
      structure SysDB =
	 struct
	    type gid = gid
	    type uid = uid

	    structure Passwd =
	       struct
		  fun name () : string = 
		    prim("Posix_SysDB_Passwd_name", ())
		  fun uid () : uid = 
		    prim("Posix_SysDB_Passwd_uid", ())
		  fun gid () : gid = 
		    prim("Posix_SysDB_Passwd_gid", ())
		  fun dir () : string = 
		    prim("Posix_SysDB_Passwd_dir", ())
		  fun shell () : string = 
		    prim("Posix_SysDB_Passwd_shell", ())
	       end

	    fun getpwnam (s:string) : bool = 
	      prim("Posix_SysDB_getpwnam", s)
	    fun getpwuid (u:uid) : bool = 
	      prim("Posix_SysDB_getpwuid", u)

	    structure Group =
	      struct
		fun name () : string = 
		  prim("Posix_SysDB_Group_name", ())
		fun gid () : gid = 
		  prim("Posix_SysDB_Group_gid", ())
		fun mem () : string list = 
		  prim("Posix_SysDB_Group_mem", ())
	       end

	    fun getgrgid (g:gid) : bool = 
	      prim("Posix_SysDB_getgrgid", g)
	    fun getgrnam (s:string) : bool = 
	      prim("Posix_SysDB_getgrnam", s)
	 end

      structure TTY =
	 struct
	    type speed = word
	    val b0 = Posix_TTY_b0_VAL: speed (* Posix_TTY_b0 *)
	    val b110 = Posix_TTY_b110_VAL: speed (* Posix_TTY_b110 *)
	    val b1200 = Posix_TTY_b1200_VAL: speed (* Posix_TTY_b1200 *)
	    val b134 = Posix_TTY_b134_VAL: speed (* Posix_TTY_b134 *)
	    val b150 = Posix_TTY_b150_VAL: speed (* Posix_TTY_b150 *)
	    val b1800 = Posix_TTY_b1800_VAL: speed (* Posix_TTY_b1800 *)
	    val b19200 = Posix_TTY_b19200_VAL: speed (* Posix_TTY_b19200 *)
	    val b200 = Posix_TTY_b200_VAL: speed (* Posix_TTY_b200 *)
	    val b2400 = Posix_TTY_b2400_VAL: speed (* Posix_TTY_b2400 *)
	    val b300 = Posix_TTY_b300_VAL: speed (* Posix_TTY_b300 *)
	    val b38400 = Posix_TTY_b38400_VAL: speed (* Posix_TTY_b38400 *)
	    val b4800 = Posix_TTY_b4800_VAL: speed (* Posix_TTY_b4800 *)
	    val b50 = Posix_TTY_b50_VAL: speed (* Posix_TTY_b50 *)
	    val b600 = Posix_TTY_b600_VAL: speed (* Posix_TTY_b600 *)
	    val b75 = Posix_TTY_b75_VAL: speed (* Posix_TTY_b75 *)
	    val b9600 = Posix_TTY_b9600_VAL: speed (* Posix_TTY_b9600 *)
	       
	    type pid = pid
	    type file_desc = file_desc

	    structure V =
	       struct
		  val eof = Posix_TTY_V_eof: int
		  val eol = Posix_TTY_V_eol: int
		  val erase = Posix_TTY_V_erase: int
		  val intr = Posix_TTY_V_intr: int
		  val kill = Posix_TTY_V_kill: int
		  val min = Posix_TTY_V_min: int
		  val nccs = Posix_TTY_V_nccs: int
		  val quit = Posix_TTY_V_quit: int
		  val start = Posix_TTY_V_start: int
		  val stop = Posix_TTY_V_stop: int
		  val susp = Posix_TTY_V_susp: int
		  val time = Posix_TTY_V_time: int
	       end

	    structure I =
	       struct
		  type flags = word
		  val brkint = Posix_TTY_I_brkint_VAL: flags (* Posix_TTY_I_brkint *)
		  val icrnl = Posix_TTY_I_icrnl_VAL: flags (* Posix_TTY_I_icrnl *)
		  val ignbrk = Posix_TTY_I_ignbrk_VAL: flags (* Posix_TTY_I_ignbrk *)
		  val igncr = Posix_TTY_I_igncr_VAL: flags (* Posix_TTY_I_igncr *)
		  val ignpar = Posix_TTY_I_ignpar_VAL: flags (* Posix_TTY_I_ignpar *)
		  val inlcr = Posix_TTY_I_inlcr_VAL: flags (* Posix_TTY_I_inlcr *)
		  val inpck = Posix_TTY_I_inpck_VAL: flags (* Posix_TTY_I_inpck *)
		  val istrip = Posix_TTY_I_istrip_VAL: flags (* Posix_TTY_I_istrip *)
		  val ixoff = Posix_TTY_I_ixoff_VAL: flags (* Posix_TTY_I_ixoff *)
		  val ixon = Posix_TTY_I_ixon_VAL: flags (* Posix_TTY_I_ixon *)
		  val parmrk = Posix_TTY_I_parmrk_VAL: flags (* Posix_TTY_I_parmrk *)
	       end

	    structure O =
	       struct
		  type flags = word
		  val opost = Posix_TTY_O_opost_VAL: flags (* Posix_TTY_O_opost *)
	       end

	    structure C =
	       struct
		  type flags = word
		  val clocal = Posix_TTY_C_clocal_VAL: flags (* Posix_TTY_C_clocal *)
		  val cread = Posix_TTY_C_cread_VAL: flags (* Posix_TTY_C_cread *)
		  val cs5 = Posix_TTY_C_cs5_VAL: flags (* Posix_TTY_C_cs5 *)
		  val cs6 = Posix_TTY_C_cs6_VAL: flags (* Posix_TTY_C_cs6 *)
		  val cs7 = Posix_TTY_C_cs7_VAL: flags (* Posix_TTY_C_cs7 *)
		  val cs8 = Posix_TTY_C_cs8_VAL: flags (* Posix_TTY_C_cs8 *)
		  val csize = Posix_TTY_C_csize_VAL: flags (* Posix_TTY_C_csize *)
		  val cstopb = Posix_TTY_C_cstopb_VAL: flags (* Posix_TTY_C_cstopb *)
		  val hupcl = Posix_TTY_C_hupcl_VAL: flags (* Posix_TTY_C_hupcl *)
		  val parenb = Posix_TTY_C_parenb_VAL: flags (* Posix_TTY_C_parenb *)
		  val parodd = Posix_TTY_C_parodd_VAL: flags (* Posix_TTY_C_parodd *)
	       end

	    structure L =
	       struct
		  type flags = word
		  val echo = Posix_TTY_L_echo_VAL: flags (* Posix_TTY_L_echo *)
		  val echoe = Posix_TTY_L_echoe_VAL: flags (* Posix_TTY_L_echoe *)
		  val echok = Posix_TTY_L_echok_VAL: flags (* Posix_TTY_L_echok *)
		  val echonl = Posix_TTY_L_echonl_VAL: flags (* Posix_TTY_L_echonl *)
		  val icanon = Posix_TTY_L_icanon_VAL: flags (* Posix_TTY_L_icanon *)
		  val iexten = Posix_TTY_L_iexten_VAL: flags (* Posix_TTY_L_iexten *)
		  val isig = Posix_TTY_L_isig_VAL: flags (* Posix_TTY_L_isig *)
		  val noflsh = Posix_TTY_L_noflsh_VAL: flags (* Posix_TTY_L_noflsh *)
		  val tostop = Posix_TTY_L_tostop_VAL: flags (* Posix_TTY_L_tostop *)
	       end

	    structure TC =
	       struct
		  type set_action = int

		  val sadrain = Posix_TTY_TC_sadrain: set_action
		  val saflush = Posix_TTY_TC_saflush: set_action
		  val sanow = Posix_TTY_TC_sanow: set_action

		  type flow_action = int

		  val ion = Posix_TTY_TC_ion: flow_action
		  val ioff = Posix_TTY_TC_ioff: flow_action
		  val ooff = Posix_TTY_TC_ooff: flow_action
		  val oon = Posix_TTY_TC_oon: flow_action

		  type queue_sel = int

		  val iflush = Posix_TTY_TC_iflush: queue_sel
		  val ioflush = Posix_TTY_TC_ioflush: queue_sel
		  val oflush = Posix_TTY_TC_oflush: queue_sel
	       end		  

	    structure Termios =
	       struct
		  type flag = word

		  fun iflag () : flag = 
		    prim("Posix_TTY_Termios_iflag", ())
		  fun oflag () : flag = 
		    prim("Posix_TTY_Termios_oflag", ())
		  fun cflag () : flag = 
		    prim("Posix_TTY_Termios_cflag", ())
		  fun lflag () : flag = 
		    prim("Posix_TTY_Termios_lflag", ())
		  fun cc () : CharArray.array = 
		    prim("Posix_TTY_Termios_cc", ())
		  fun setcc (cc:CharArray.array) : unit = 
		    prim("Posix_TTY_Termios_setcc", cc)
		  fun ospeed () : speed =
		    prim("Posix_TTY_Termios_cfgetospeed", ())
		  fun ispeed () : speed =
		    prim("Posix_TTY_Termios_cfgetispeed", ())
		  fun setiflag (f:flag) : unit =
		    prim("Posix_TTY_Termios_setiflag", f)
		  fun setoflag (f:flag) : unit =
		    prim("Posix_TTY_Termios_setoflag", f)
		  fun setcflag (f:flag) : unit =
		    prim("Posix_TTY_Termios_setcflag", f)
		  fun setlflag (f:flag) : unit =
		    prim("Posix_TTY_Termios_setlflag", f)
		  fun setospeed (s:speed) : int =
		    prim("Posix_TTY_Termios_setospeed", s)
		  fun setispeed (s:speed) : int =
		    prim("Posix_TTY_Termios_setispeed", s)
	       end

	    fun getattr (fd:fd) : int =
	      prim("Posix_TTY_getattr", fd)
	    fun setattr (fd:fd,sa:TC.set_action) : int =
	      prim("Posix_TTY_setattr", (fd,sa))
	    fun sendbreak (fd:fd,i:int) : int = prim("@tcsendbreak", (fd,i))
	    fun drain (fd:fd) : int = prim("@tcdrain", fd)
	    fun flush (fd:fd,qs:TC.queue_sel) : int = prim("@tcflush", (fd,qs))
	    fun flow (fd:fd,fa:TC.flow_action) : int = prim("@tcflow", (fd,fa))
	    fun getpgrp (fd:fd) : pid = prim("@tcgetpgrp", fd)
	    fun setpgrp (fd:fd,pid:pid) : int = prim("@tcsetpgrp", (fd,pid))
	 end
   end
