structure PosixProcEnv: POSIX_PROC_ENV =
   struct
      structure Prim = PosixPrimitive.ProcEnv
      structure Error = PosixError
      structure CString = PosixPrimitive.CString

      local open Prim
      in
	 type pid = pid
	 type uid = uid
	 type gid = gid
	 type file_desc = file_desc
	 val getpgrp = getpgrp (* No error checking required *)
	 val getegid = getegid (* No error checking required *)
	 val geteuid = geteuid (* No error checking required *)
	 val getgid = getgid (* No error checking required *)
	 val getpid = getpid (* No error checking required *)
	 val getppid = getppid (* No error checking required *)
	 val getuid = getuid (* No error checking required *)
	 val getgroups = getgroups
	 val setgid = fn a => (Error.checkResult o setgid) a
	 val setsid = fn a => (Error.checkReturnResult o setsid) a
	 val setuid = fn a => (Error.checkResult o setuid) a
      end

      fun id x = x
      val uidToWord = id 
      val wordToUid = id
      val gidToWord = id
      val wordToGid = id

      fun getlogin () =
	 let val s = Prim.getlogin ()
	 in if CString.isNull s
	       then raise (Error.SysErr ("no login name", NONE))
	    else CString.toString s
	 end

      fun setpgid {pid, pgid} =
	 let
	    val f =
	       fn NONE => 0
		| SOME n => n
	 in Error.checkResult (Prim.setpgid (f pid, f pgid))
	 end

      local
	 structure Uname = Prim.Uname
      in
	 fun uname () =
	    (Error.checkResult (Uname.uname ());
	     [("sysname", Uname.sysname ()),
	      ("nodename", Uname.nodename ()),
	      ("release", Uname.release ()),
	      ("version", Uname.version ()),
	      ("machine", Uname.machine ())])
      end

      val time = Time.now

      fun sysconf s =
	case Prim.sysconfNameLookup s of
	    NONE => Error.raiseSys Error.inval
	  | SOME n => let val res = Prim.sysconf n
		      in Error.checkResult res;
			SysWord.fromInt res
		      end
	       
      local
	 structure Tms = Prim.Tms

(*
	 val ticksPerSecond: LargeInt.int =
	    SysWord.toLargeInt (sysconf "CLK_TCK")

	 val millisecondsPerSecond: LargeInt.int = 1000
	    
	 fun cvt (ticks: int): Time.time =
	    Time.fromMilliseconds
	    (LargeInt.div
	     (LargeInt.fromInt ticks * millisecondsPerSecond,
	      ticksPerSecond))
*)
	 fun ticksPerSec () = Real.fromInt (SysWord.toIntX (sysconf "CLK_TCK"))
	 
	 fun cvt (ticks: word) =
	    Time.fromReal (real(Word.toInt ticks) / ticksPerSec())
      in
	 fun times () =
	    let
	       val elapsed = Prim.times ()
	    in
	       {elapsed = cvt elapsed,
		utime = cvt (Tms.utime ()), 
		stime = cvt (Tms.stime ()), 
		cutime = cvt (Tms.cutime ()), 
		cstime = cvt (Tms.cstime ())}
	    end
      end

      val environ = Prim.environ

      fun getenv name =
	 let val cs = Prim.getenv name
	 in if CString.isNull cs then NONE
	    else SOME (CString.toString cs)
	 end

      fun ctermid () = Prim.ctermid ()

      fun isatty n = Prim.isatty n

      fun ttyname n =
	 let val cs = Prim.ttyname n
	 in if CString.isNull cs
	       then Error.error ()
	    else CString.toString cs
	 end
   end
