structure PosixSysDB: POSIX_SYS_DB =
  struct
    structure Prim = PosixPrimitive.SysDB
    structure Error = PosixError

    type uid = Prim.uid
    type gid = Prim.gid

    structure Passwd =
      struct
	type passwd = {name: string,
		       uid: uid,
		       gid: gid,
		       home: string,
		       shell: string}

	local
	  structure C = Prim.Passwd
	in
	  fun fromC(b: bool): passwd =
	    if b then {name = C.name(),
		       uid = C.uid(),
		       gid = C.gid(),
		       home = C.dir(),
		       shell = C.shell()}
	    else Error.error()
	end

	fun name (p: passwd) : string = #name p
	fun uid (p: passwd) : uid = #uid p
	fun gid (p: passwd) : gid = #gid p
	fun home (p: passwd) : string = #home p
	fun shell (p: passwd) : string = #shell p 
      end

    fun getpwnam a = (Passwd.fromC o Prim.getpwnam) a
    fun getpwuid a = (Passwd.fromC o Prim.getpwuid) a
   
    structure Group =
      struct
	type group = {name: string,
		      gid: gid,
		      members: string list}

	structure Group = Prim.Group

	fun fromC(b: bool): group =
	  if b then {name = Group.name(),
		     gid = Group.gid(),
		     members = Group.mem()}
	  else Error.error()
	    
	fun name (g: group) : string = #name g
	fun gid (g: group) : gid = #gid g
	fun members (g: group) : string list = #members g
      end

    fun getgrnam a = (Group.fromC o Prim.getgrnam) a
    fun getgrgid a = (Group.fromC o Prim.getgrgid) a
  end
