structure PosixTTY: POSIX_TTY =
  struct
    structure Prim = PosixPrimitive.TTY
    open Prim
    structure Error = PosixError
      
    type pid = Prim.pid
    type file_desc = Prim.file_desc

    structure V =
      struct
	open V	  
	type cc = CharArray.array

	val default = #"\000"

	fun new () = CharArray.array (nccs, default)

	fun updates (a, l) = List.app (fn (i, c) => CharArray.update (a, i, c)) l

	fun cc l = let val a = new ()
		   in updates (a, l)
		     ; a
		   end

	fun update (a, l) =
	  let val a' = new ()
	  in CharArray.copy {src = a, si = 0, len = NONE,
			 dst = a', di = 0}
	    ; updates (a', l)
	    ; a'
	  end
	
	fun sub a = CharArray.sub a
      end
      
    structure I = struct open I PosixFlags end
    structure O = struct open O PosixFlags end
    structure C = struct open C PosixFlags end
    structure L = struct open L PosixFlags end

    type speed = Prim.speed
      
    val compareSpeed = SysWord.compare
    fun id x = x
    val speedToWord = id
    val wordToSpeed = id

    type termios = {iflag: I.flags,
		    oflag: O.flags,
		    cflag: C.flags,
		    lflag: L.flags,
		    cc: V.cc,
		    ispeed: speed,
		    ospeed: speed}

    val termios = id
    val fieldsOf = id
      
    fun getiflag (t: termios) : I.flags = #iflag t
    fun getoflag (t: termios) : O.flags = #oflag t
    fun getcflag (t: termios) : C.flags = #cflag t
    fun getlflag (t: termios) : L.flags = #oflag t
    fun getcc    (t: termios) : V.cc = #cc t

    structure CF =
      struct
	fun getospeed (t: termios) : speed = #ospeed t
	fun setospeed ({iflag, oflag, cflag, lflag, cc, ispeed, ...}: termios,
		       ospeed: speed): termios =
	  {iflag = iflag,
	   oflag = oflag,
	   cflag = cflag,
	   lflag = lflag,
	   cc = cc,
	   ispeed = ispeed,
	   ospeed = ospeed}
		
	fun getispeed (t: termios) : speed = #ispeed t
	  
	fun setispeed ({iflag, oflag, cflag, lflag, cc, ospeed, ...}: termios,
		       ispeed: speed): termios =
	  {iflag = iflag,
	   oflag = oflag,
	   cflag = cflag,
	   lflag = lflag,
	   cc = cc,
	   ispeed = ispeed,
	   ospeed = ospeed}
      end
    
    structure Termios = Prim.Termios
      
    fun getattr fd =
      (Error.checkResult (Prim.getattr fd)
       ; {iflag = Termios.iflag (),
	  oflag = Termios.oflag (),
	  cflag = Termios.cflag (),
	  lflag = Termios.lflag (),
	  cc = Termios.cc (),
	  ispeed = Termios.ispeed (),
	  ospeed = Termios.ospeed ()})
      
    fun setattr (fd, a, {iflag, oflag, cflag, lflag, cc, ispeed, ospeed}) =
      (Termios.setiflag iflag
       ; Termios.setoflag oflag
       ; Termios.setcflag cflag
       ; Termios.setlflag lflag
       ; Termios.setospeed ospeed
       ; Termios.setispeed ispeed
       ; Termios.setcc cc 
       ; Error.checkResult (Prim.setattr (fd, a)))
      
    fun sendbreak (fd, n) =
      Error.checkResult (Prim.sendbreak (fd, n))
      
    fun drain (fd) = Error.checkResult (Prim.drain fd)
      
    fun flush (fd, n) = Error.checkResult (Prim.flush (fd, n))
      
    fun flow (fd, n) = Error.checkResult (Prim.flow (fd, n))
	
    fun getpgrp (fd) = Error.checkReturnResult (Prim.getpgrp fd)
	
    fun setpgrp (fd, pid) = Error.checkResult (Prim.setpgrp (fd, pid))
  end
